@@ -29,6 +29,14 @@
 
 #include <kde_file.h>
 
+#include <KSaveFile>
+
+#if 0
+#define EDIT_DEBUG kDebug()
+#else
+#define EDIT_DEBUG if (0) kDebug()
+#endif
+
 namespace Kate {
 
 TextBuffer::TextBuffer (KateDocument *parent, int blockSize)
@@ -49,7 +57,6 @@ TextBuffer::TextBuffer (KateDocument *parent, int blockSize)
   , m_textCodec (0)
   , m_generateByteOrderMark (false)
   , m_endOfLineMode (eolUnix)
-  , m_removeTrailingSpaces (false)
   , m_newLineAtEof (false)
   , m_lineLengthLimit (1024)
 {
@@ -208,6 +215,9 @@ bool TextBuffer::finishEditing ()
 
 void TextBuffer::wrapLine (const KTextEditor::Cursor &position)
 {
+  // debug output for REAL low-level debugging
+  EDIT_DEBUG << "wrapLine" << position;
+
   // only allowed if editing transaction running
   Q_ASSERT (m_editingTransactions > 0);
 
@@ -244,6 +254,9 @@ void TextBuffer::wrapLine (const KTextEditor::Cursor &position)
 
 void TextBuffer::unwrapLine (int line)
 {
+  // debug output for REAL low-level debugging
+  EDIT_DEBUG << "unwrapLine" << line;
+
   // only allowed if editing transaction running
   Q_ASSERT (m_editingTransactions > 0);
 
@@ -290,6 +303,9 @@ void TextBuffer::unwrapLine (int line)
 
 void TextBuffer::insertText (const KTextEditor::Cursor &position, const QString &text)
 {
+  // debug output for REAL low-level debugging
+  EDIT_DEBUG << "insertText" << position << text;
+
   // only allowed if editing transaction running
   Q_ASSERT (m_editingTransactions > 0);
 
@@ -319,6 +335,9 @@ void TextBuffer::insertText (const KTextEditor::Cursor &position, const QString
 
 void TextBuffer::removeText (const KTextEditor::Range &range)
 {
+  // debug output for REAL low-level debugging
+  EDIT_DEBUG << "removeText" << range;
+
   // only allowed if editing transaction running
   Q_ASSERT (m_editingTransactions > 0);
 
@@ -360,37 +379,54 @@ int TextBuffer::blockForLine (int line) const
   if ((line < 0) || (line >= lines()))
     qFatal ("out of range line requested in text buffer (%d out of [0, %d[)", line, lines());
 
-  // block to start search with
-  int index = m_lastUsedBlock;
+  // we need blocks and last used block should not be negative
+  Q_ASSERT (!m_blocks.isEmpty());
+  Q_ASSERT (m_lastUsedBlock >= 0);
+  
+  /**
+   * shortcut: try last block first
+   */
+  if (m_lastUsedBlock < m_blocks.size()) {
+    /**
+     * check if block matches
+     * if yes, just return again this block
+     */
+    TextBlock* block = m_blocks[m_lastUsedBlock];
+    const int start = block->startLine();
+    const int lines = block->lines ();
+    if (start <= line && line < (start + lines))
+      return m_lastUsedBlock;
+  }
+
+  /**
+   * search for right block
+   * use binary search
+   * if we leave this loop not by returning the found element we have an error
+   */
   int blockStart = 0;
-  int blockEnd = m_blocks.size();
+  int blockEnd = m_blocks.size() - 1;
+  while (blockEnd >= blockStart) {
+    // get middle and ensure it is OK
+    int middle = blockStart + ((blockEnd - blockStart) / 2);
+    Q_ASSERT (middle >= 0);
+    Q_ASSERT (middle < m_blocks.size());
 
-  // check if start is ok
-  if (index < 0 || index >= m_blocks.size())
-    index = 0;
-
-  // search for right block
-  forever {
     // facts bout this block
-    TextBlock* block = m_blocks.at(index);
+    TextBlock* block = m_blocks[middle];
     const int start = block->startLine();
     const int lines = block->lines ();
 
     // right block found, remember it and return it
     if (start <= line && line < (start + lines)) {
-      m_lastUsedBlock = index;
-      return index;
+      m_lastUsedBlock = middle;
+      return middle;
     }
 
+    // half our stuff ;)
     if (line < start)
-	{
-	  // Search left of index
-	  blockEnd = index;
-	}else{
-	  // Search right of index
-	  blockStart = index+1;
-	}
-	index = (blockStart + (blockEnd-1)) / 2;
+      blockEnd = middle - 1;
+    else
+      blockStart = middle + 1;
   }
 
   // we should always find a block
@@ -472,7 +508,7 @@ void TextBuffer::debugPrint (const QString &title) const
     m_blocks.at(i)->debugPrint (i);
 }
 
-bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooLongLinesWrapped)
+bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooLongLinesWrapped, bool enforceTextCodec)
 {
   // fallback codec must exist
   Q_ASSERT (m_fallbackTextCodec);
@@ -504,7 +540,7 @@ bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooL
    * 2) use fallback encoding, be done, if no encoding errors happen
    * 3) use again given encoding, be done in any case
    */
-  for (int i = 0; i < 4;  ++i) {
+  for (int i = 0; i < (enforceTextCodec ? 1 : 4);  ++i) {
     /**
      * kill all blocks beside first one
      */
@@ -550,7 +586,7 @@ bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooL
       encodingErrors = encodingErrors || currentError;
 
       // bail out on encoding error, if not last round!
-      if (encodingErrors && i < 3) {
+      if (encodingErrors && i < (enforceTextCodec ? 0 : 3)) {
         kDebug (13020) << "Failed try to load file" << filename << "with codec" <<
                           (file.textCodec() ? file.textCodec()->name() : "(null)");
         break;
@@ -624,6 +660,9 @@ bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooL
     }
   }
 
+  // save md5sum of file on disk
+  setDigest (file.digest ());
+
   // remember if BOM was found
   if (file.byteOrderMarkFound ())
     setGenerateByteOrderMark (true);
@@ -655,6 +694,16 @@ bool TextBuffer::load (const QString &filename, bool &encodingErrors, bool &tooL
   return true;
 }
 
+const QByteArray &TextBuffer::digest () const
+{
+  return m_digest;
+}
+
+void TextBuffer::setDigest (const QByteArray & md5sum)
+{
+  m_digest = md5sum;
+}
+
 void TextBuffer::setTextCodec (QTextCodec *codec) 
 {
   m_textCodec = codec;
@@ -671,14 +720,30 @@ bool TextBuffer::save (const QString &filename)
 {
   // codec must be set!
   Q_ASSERT (m_textCodec);
+  
+  /**
+   * use KSaveFile for save write + rename
+   */
+  KSaveFile saveFile (filename);
+  if (!saveFile.open())
+    return false;
 
   /**
    * construct correct filter device and try to open
    */
-  QIODevice *file = KFilterDev::deviceForFile (filename, m_mimeTypeForFilterDev, false);
-  if (!file->open (QIODevice::WriteOnly)) {
-    delete file;
-    return false;
+  QIODevice *file = KFilterDev::device (&saveFile, m_mimeTypeForFilterDev, false);
+  const bool deleteFile = file;
+  if (!file)
+    file = &saveFile;
+
+  /**
+   * try to open, if new file
+   */
+  if (deleteFile) {
+    if (!file->open (QIODevice::WriteOnly)) {
+      delete file;
+      return false;
+    }
   }
 
   /**
@@ -717,7 +782,7 @@ bool TextBuffer::save (const QString &filename)
     Q_ASSERT(m_lines > 0); // see .h file
     const Kate::TextLine lastLine = line (m_lines - 1);
     const int firstChar = lastLine->firstChar();
-    if (firstChar > -1 || (lastLine->length() > 0 && !m_removeTrailingSpaces)) {
+    if (firstChar > -1 || lastLine->length() > 0) {
       stream << eol;
     }
   }
@@ -726,24 +791,27 @@ bool TextBuffer::save (const QString &filename)
   stream.flush ();
 
   // close and delete file
-  file->close ();
-  delete file;
+  if (deleteFile) {
+    file->close ();
+    delete file;
+  }
+  
+  // flush file
+  if (!saveFile.flush())
+    return false;
 
 #ifndef Q_OS_WIN
   // ensure that the file is written to disk
-  // we crete new qfile, as the above might be wrapper around compression
-  QFile syncFile (filename);
-  syncFile.open (QIODevice::ReadOnly);
-
 #ifdef HAVE_FDATASYNC
-  fdatasync (syncFile.handle());
+  fdatasync (saveFile.handle());
 #else
-  fsync (syncFile.handle());
+  fsync (saveFile.handle());
 #endif
 #endif
-
+  
   // did save work?
-  bool ok = stream.status() == QTextStream::Ok;
+  // only finalize if stream status == OK
+  bool ok = (stream.status() == QTextStream::Ok) && saveFile.finalize();
 
   // remember this revision as last saved if we had success!
   if (ok)
