@@ -119,10 +119,9 @@ public Q_SLOTS:
     void enterPowerMode();
     void enterIncrementalMode();
 
-    bool hideInfoMessage();
     bool clearHighlights();
     void updateHighlightColors();
-    
+
     // read write status of document changed
     void slotReadWriteChanged ();
 
@@ -146,6 +145,8 @@ private Q_SLOTS:
     void onPowerReplacmentContextMenuRequest();
     void onPowerReplacmentContextMenuRequest(const QPoint&);
 
+    void keepHighlights();
+
 private:
     // Helpers
     bool find(SearchDirection searchDirection = SearchForward, const QString * replacement = 0);
@@ -170,6 +171,8 @@ private:
     void sendConfig();
     void fixForSingleLine(KTextEditor::Range & range, SearchDirection searchDirection);
 
+    void showInfoMessage(const QString& text);
+
 private:
     KateView *const m_view;
     KateViewConfig *const m_config;
